import numpy as np
import matplotlib.pyplot as plt

# The two given datasets, assuming both were generated from the same distribution,
# so X_i ~ Po(lambda) and Y_i ~ Po(lambda).
x = [3, 1, 5]
y = [2, 3]

# MLE estimate for lambda.
lambda_hat = np.mean(x + y)

# Estimating the quantity of interest.
def t(x,y):
    return np.mean(y) - np.mean(x)
# Resampling from the common Poisson distribution.
def rxy0():
    return (np.random.poisson(np.mean(x+y), size=len(x)),
            np.random.poisson(np.mean(x+y), size=len(y)))

t0_ = np.array([t(*rxy0()) for _ in range(10000)])
# Note: t0_ >= t(x, y) is a boolean. Taking the mean over the entire
# numpy array is equivalent to finding the proportion.
p = 2 * min(np.mean(t0_ >= t(x, y)), np.mean(t0_ <= t(x, y)))

print(f"p-value: {p}")
# One possible output:
# p-value: 0.7936
