import numpy as np
import pandas
import matplotlib.pyplot as plt
import sklearn.linear_model

# Load the dataset
url = 'https://www.cl.cam.ac.uk/teaching/2021/DataSci/data/climate.csv'
climate = pandas.read_csv(url)
climate = climate.loc[(climate.station == 'Cambridge') & (climate.yyyy >= 1985)].copy()
t = climate.yyyy + (climate.mm-1)/12
temp = (climate.tmin + climate.tmax)/2

# Fit a simple linear model.
X = np.column_stack([np.sin(2 * np.pi * t), np.cos(2 * np.pi * t), t-2000])
model = sklearn.linear_model.LinearRegression()
model.fit(X, temp)

# Extend the time scale.
tnew = np.linspace(2010, 2025, (2025-202)*12+1)
Xnew = np.column_stack([np.sin(2*np.pi*tnew), np.cos(2*np.pi*tnew), tnew-2000])
tempnew = model.predict(Xnew)

# Step 1. Define function that generates model from samples.
def get_model(temp):
    new_model = sklearn.linear_model.LinearRegression()
    new_model.fit(X, temp)
    return new_model


# Define a function to generate a synthetic copy of the dataset.
pred = model.predict(X)
sigma_hat = np.sqrt(np.mean((pred - temp) ** 2))
def rtemp():
    return np.random.normal(loc=pred, scale=sigma_hat)

# Step 3. Sample the models and for each model make predictions for Xnew.
models_ = [get_model(rtemp()) for _ in range(10000)]
preds = np.column_stack(np.array([cur_model.predict(Xnew) for cur_model in models_]))
# Step 4. For each x in Xnew, compute the quantile of interest.
# Note: It would be more intuitive to do this using a for-loop,
# but it would be slower.
conf_interval = np.quantile(preds, [0.025, 0.975], axis=1)
lows = conf_interval[0, :]
highs = conf_interval[1, :]
