import numpy as np
import scipy.stats

sigma = 0.23
num_samples = 1000
num_bins = 50
phi_values = [scipy.stats.norm.cdf(i) for i in np.linspace(-2.5, 2.5, num_bins)]
x = np.random.normal(loc=0, scale=sigma, size=num_samples) # np.random.poisson(0.4, size=num_samples).astype(float)

sigma_hat = np.sqrt(np.mean(x ** 2))


def ell1_distance(samples):
    # This computes how many values are below each i in np.linspace(..)
    # We do this efficiently by first sorting all samples and then
    # counting all values below each threshold.
    samples /= sigma_hat
    np.sort(samples)
    cur = 0
    acc = np.zeros(num_bins)
    sm = 0
    idx = 0
    for i in np.linspace(-2.5, 2.5, num_bins):
        while cur < len(samples) and samples[cur] < i:
            sm += samples[cur]
            cur += 1
        acc[idx] = sm
        idx += 1
    acc /= len(samples)
    return np.linalg.norm((phi_values - acc), ord=1)


distances = np.array([ell1_distance(np.random.normal(loc=0, scale=sigma_hat, size=num_samples)) for _ in range(1000)])
lo, hi = np.quantile(distances, [0.025, 0.975])
print(f"L1 distance received: {ell1_distance(x)}")
print(f"L1 distance conf interval: [{lo}, {hi}]")
# One possible output:
# L1 distance received: 24.992...
# L1 distance conf interval: [24.917..., 25.252...]
